# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Sputnixusp(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.ax25_header.pid
    :field payload___packet_type: ax25_frame.payload.___.packet_type
    :field t_amp: ax25_frame.payload.0.tlm.t_amp
    :field t_uhf: ax25_frame.payload.0.tlm.t_uhf
    :field rssi_rx: ax25_frame.payload.0.tlm.rssi_rx
    :field pf: ax25_frame.payload.0.tlm.pf
    :field pb: ax25_frame.payload.0.tlm.pb
    :field nres_uhf: ax25_frame.payload.0.tlm.nres_uhf
    :field fl_uhf: ax25_frame.payload.0.tlm.fl_uhf
    :field time_uhf: ax25_frame.payload.0.tlm.time_uhf
    :field uptime_uhf: ax25_frame.payload.0.tlm.uptime_uhf
    :field current_uhf: ax25_frame.payload.0.tlm.current_uhf
    :field uuhf: ax25_frame.payload.0.tlm.uuhf
    :field rssi_idle: ax25_frame.payload.0.tlm.rssi_idle
    :field rxbitrate: ax25_frame.payload.0.tlm.rxbitrate
    :field num_active_schedules: ax25_frame.payload.0.tlm.num_active_schedules
    :field reset_during_sch: ax25_frame.payload.0.tlm.reset_during_sch
    :field backup_sch_active: ax25_frame.payload.0.tlm.backup_sch_active
    :field usb1: ax25_frame.payload.1.tlm.usb1
    :field usb2: ax25_frame.payload.1.tlm.usb2
    :field usb3: ax25_frame.payload.1.tlm.usb3
    :field isb1: ax25_frame.payload.1.tlm.isb1
    :field isb2: ax25_frame.payload.1.tlm.isb2
    :field isb3: ax25_frame.payload.1.tlm.isb3
    :field iab: ax25_frame.payload.1.tlm.iab
    :field ich1: ax25_frame.payload.1.tlm.ich1
    :field ich2: ax25_frame.payload.1.tlm.ich2
    :field ich3: ax25_frame.payload.1.tlm.ich3
    :field ich4: ax25_frame.payload.1.tlm.ich4
    :field ich5: ax25_frame.payload.1.tlm.ich5
    :field t1_pw: ax25_frame.payload.1.tlm.t1_pw
    :field t2_pw: ax25_frame.payload.1.tlm.t2_pw
    :field t3_pw: ax25_frame.payload.1.tlm.t3_pw
    :field t4_pw: ax25_frame.payload.1.tlm.t4_pw
    :field flags1: ax25_frame.payload.1.tlm.flags1
    :field flags2: ax25_frame.payload.1.tlm.flags2
    :field flags3: ax25_frame.payload.1.tlm.flags3
    :field reserved1: ax25_frame.payload.1.tlm.reserved1
    :field uab: ax25_frame.payload.1.tlm.uab
    :field reg_tel_id: ax25_frame.payload.1.tlm.reg_tel_id
    :field time: ax25_frame.payload.1.tlm.time
    :field nres_ps: ax25_frame.payload.1.tlm.nres_ps
    :field fl_ps: ax25_frame.payload.1.tlm.fl_ps
    :field uab1: ax25_frame.payload.1.tlm.uab1
    :field uab2: ax25_frame.payload.1.tlm.uab2
    :field capacity: ax25_frame.payload.1.tlm.capacity
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Sputnixusp.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Sputnixusp.Ax25Header(self._io, self, self._root)
            self.payload = []
            i = 0
            while not self._io.is_eof():
                self.payload.append(Sputnixusp.BeaconTlm(self._io, self, self._root))
                i += 1



    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Sputnixusp.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Sputnixusp.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Sputnixusp.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Sputnixusp.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class RegularTelemetry6u(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.usb1 = self._io.read_u2le()
            self.usb2 = self._io.read_u2le()
            self.usb3 = self._io.read_u2le()
            self.isb1 = self._io.read_u2le()
            self.isb2 = self._io.read_u2le()
            self.isb3 = self._io.read_u2le()
            self.iab = self._io.read_s2le()
            self.uab = self._io.read_u2le()
            self.uab1 = self._io.read_u2le()
            self.uab2 = self._io.read_u2le()
            self.t1_pw = self._io.read_s2le()
            self.t2_pw = self._io.read_s2le()
            self.capacity = self._io.read_u1()
            self.channel_status_ch1 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch2 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch3 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch4 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch5 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch6 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch7 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch8 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch9 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch10 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch11 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch12 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch13 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch14 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch15 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch16 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch17 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch18 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch19 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch20 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch21 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch22 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch23 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch24 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch25 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch26 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch27 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch28 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch29 = self._io.read_bits_int_be(1) != 0
            self.channel_status_ch30 = self._io.read_bits_int_be(1) != 0
            self.channel_status_reserved = self._io.read_bits_int_be(2)
            self._io.align_to_byte()
            self.flags1 = self._io.read_u1()
            self.flags2 = self._io.read_u1()
            self.flags3 = self._io.read_u1()
            self.reserved1 = self._io.read_u1()
            self.reg_tel_id = self._io.read_u4le()
            self.time = self._io.read_u4le()
            self.nres_ps = self._io.read_u1()
            self.fl_ps = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class UhfBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.t_amp = self._io.read_s1()
            self.t_uhf = self._io.read_s1()
            self.rss_rx = self._io.read_u1()
            self.rssi_idle = self._io.read_u1()
            self.pf = self._io.read_s1()
            self.pb = self._io.read_s1()
            self.nres_uhf = self._io.read_u1()
            self.fl_uhf = self._io.read_u1()
            self.time_uhf = self._io.read_u4le()
            self.uptime_uhf = self._io.read_u4le()
            self.rxbitrate = self._io.read_u4le()
            self.current_uhf = self._io.read_u2le()
            self.uuhf = self._io.read_u2le()
            self.sch_reserved = self._io.read_bits_int_be(1) != 0
            self.backup_sch_active = self._io.read_bits_int_be(1) != 0
            self.reset_during_sch = self._io.read_bits_int_be(1) != 0
            self.num_active_schedules = self._io.read_bits_int_be(5)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class BeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_type = self._io.read_u2le()
            self.skip1 = self._io.read_u2le()
            self.skip2 = self._io.read_u2le()
            self.len = self._io.read_u2le()
            _on = self.packet_type
            if _on == 60705:
                self._raw_tlm = self._io.read_bytes(self.len)
                _io__raw_tlm = KaitaiStream(BytesIO(self._raw_tlm))
                self.tlm = Sputnixusp.PsRegularTelemetry5ch(_io__raw_tlm, self, self._root)
            elif _on == 16966:
                self._raw_tlm = self._io.read_bytes(self.len)
                _io__raw_tlm = KaitaiStream(BytesIO(self._raw_tlm))
                self.tlm = Sputnixusp.UhfBeacon(_io__raw_tlm, self, self._root)
            elif _on == 56865:
                self._raw_tlm = self._io.read_bytes(self.len)
                _io__raw_tlm = KaitaiStream(BytesIO(self._raw_tlm))
                self.tlm = Sputnixusp.PsRegularTelemetry(_io__raw_tlm, self, self._root)
            elif _on == 16918:
                self._raw_tlm = self._io.read_bytes(self.len)
                _io__raw_tlm = KaitaiStream(BytesIO(self._raw_tlm))
                self.tlm = Sputnixusp.GeneralTlm(_io__raw_tlm, self, self._root)
            elif _on == 57125:
                self._raw_tlm = self._io.read_bytes(self.len)
                _io__raw_tlm = KaitaiStream(BytesIO(self._raw_tlm))
                self.tlm = Sputnixusp.RegularTelemetry6u(_io__raw_tlm, self, self._root)
            else:
                self.tlm = self._io.read_bytes(self.len)


    class PsRegularTelemetry(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.usb1 = self._io.read_u2le()
            self.usb2 = self._io.read_u2le()
            self.usb3 = self._io.read_u2le()
            self.isb1 = self._io.read_u2le()
            self.isb2 = self._io.read_u2le()
            self.isb3 = self._io.read_u2le()
            self.iab = self._io.read_s2le()
            self.ich1 = self._io.read_u2le()
            self.ich2 = self._io.read_u2le()
            self.ich3 = self._io.read_u2le()
            self.ich4 = self._io.read_u2le()
            self.t1_pw = self._io.read_s2le()
            self.t2_pw = self._io.read_s2le()
            self.t3_pw = self._io.read_s2le()
            self.t4_pw = self._io.read_s2le()
            self.flags1 = self._io.read_u1()
            self.flags2 = self._io.read_u1()
            self.flags3 = self._io.read_u1()
            self.reserved1 = self._io.read_u1()
            self.uab = self._io.read_u2le()
            self.reg_tel_id = self._io.read_u4le()
            self.time = self._io.read_u4le()
            self.nres_ps = self._io.read_u1()
            self.fl_ps = self._io.read_u1()


    class PsRegularTelemetry5ch(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.usb1 = self._io.read_u2le()
            self.usb2 = self._io.read_u2le()
            self.usb3 = self._io.read_u2le()
            self.isb1 = self._io.read_u2le()
            self.isb2 = self._io.read_u2le()
            self.isb3 = self._io.read_u2le()
            self.iab = self._io.read_s2le()
            self.ich1 = self._io.read_u2le()
            self.ich2 = self._io.read_u2le()
            self.ich3 = self._io.read_u2le()
            self.ich4 = self._io.read_u2le()
            self.ich5 = self._io.read_u2le()
            self.t1_pw = self._io.read_s2le()
            self.t2_pw = self._io.read_s2le()
            self.t3_pw = self._io.read_s2le()
            self.t4_pw = self._io.read_s2le()
            self.flags1 = self._io.read_u1()
            self.flags2 = self._io.read_u1()
            self.flags3 = self._io.read_u1()
            self.reserved1 = self._io.read_u1()
            self.uab = self._io.read_u2le()
            self.reg_tel_id = self._io.read_u4le()
            self.time = self._io.read_u4le()
            self.nres_ps = self._io.read_u1()
            self.fl_ps = self._io.read_u1()


    class GeneralTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ps = Sputnixusp.PsRegularTelemetry(self._io, self, self._root)
            self.t_amp = self._io.read_u1()
            self.t_uhf = self._io.read_u1()
            self.rssi_rx = self._io.read_s2be()
            self.pf = self._io.read_u1()
            self.pb = self._io.read_u1()
            self.nres_uhf = self._io.read_u1()
            self.fl_uhf = self._io.read_u1()
            self.time_uhf = self._io.read_u4le()
            self.uptime_uhf = self._io.read_u4le()
            self.current_uhf = self._io.read_u2le()
            self.uuhf = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Sputnixusp.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Estcube2(KaitaiStruct):
    """:field timestamp: icp_data.eps.common.unix_time
    :field eps_uptime: icp_data.eps.common.uptime
    :field eps_cpu_temperature: icp_data.eps.common.cpu_temperature
    :field eps_bus_voltage: icp_data.eps.bus_voltage
    :field eps_avg_power_balance: icp_data.eps.avg_power_balance
    :field eps_enable_st: icp_data.eps.enable_st
    :field eps_enable_obc: icp_data.eps.enable_obc
    :field eps_enable_com3v3: icp_data.eps.enable_com_3v3
    :field eps_enable_comdcdc: icp_data.eps.enable_com_dcdc
    :field eps_enable_com_pa: icp_data.eps.enable_com_pa
    :field eps_enable_eps_dcdc1: icp_data.eps.enable_eps_dcdc1
    :field eps_enable_eps_dcdc2: icp_data.eps.enable_eps_dcdc2
    :field eps_enable_sp: icp_data.eps.enable_sp
    :field eps_enable_cgp: icp_data.eps.enable_cgp
    :field eps_enable_hscom: icp_data.eps.enable_hscom
    :field eps_enable_eop1: icp_data.eps.enable_eop1
    :field eps_enable_eop2: icp_data.eps.enable_eop2
    :field eps_enable_esail: icp_data.eps.enable_esail
    :field eps_enable_rw: icp_data.eps.enable_obc_rw
    
    :field eps_battery_current_a: icp_data.eps.battery_current_a
    :field eps_battery_current_b: icp_data.eps.battery_current_b
    :field eps_battery_voltage_a: icp_data.eps.battery_voltage_a
    :field eps_battery_voltage_b: icp_data.eps.battery_voltage_b
    :field eps_battery_voltage_c: icp_data.eps.battery_voltage_c
    :field eps_battery_voltage_d: icp_data.eps.battery_voltage_d
    :field eps_battery_temperature_a: icp_data.eps.battery_temperature_a
    :field eps_battery_temperature_b: icp_data.eps.battery_temperature_b
    :field eps_battery_temperature_c: icp_data.eps.battery_temperature_c
    :field eps_battery_temperature_d: icp_data.eps.battery_temperature_d
    
    :field eps_st_voltage: icp_data.eps.st_voltage
    :field eps_st_current: icp_data.eps.st_current
    :field eps_obc_voltage: icp_data.eps.obc_voltage
    :field eps_obc_current: icp_data.eps.obc_current
    :field eps_com_3v3_voltage: icp_data.eps.com_3v3_voltage
    :field eps_com_pa_voltage: icp_data.eps.com_pa_voltage
    :field eps_com_3v3_current: icp_data.eps.com_3v3_current
    :field eps_com_pa_current: icp_data.eps.com_pa_current
    :field eps_eps_voltage: icp_data.eps.eps_voltage
    :field eps_eps_dcdc_3v3_voltage: icp_data.eps.eps_dcdc_3v3_voltage
    :field eps_eps_current: icp_data.eps.eps_current
    :field eps_eps_dcdc1_current: icp_data.eps.eps_dcdc1_current
    :field eps_eps_dcdc2_current: icp_data.eps.eps_dcdc2_current
    :field eps_sp_voltage: icp_data.eps.sp_voltage
    :field eps_sp_current: icp_data.eps.sp_current
    :field eps_cpd_voltage: icp_data.eps.cpd_voltage
    :field eps_cpd_current: icp_data.eps.cpd_current
    :field eps_cam1_voltage: icp_data.eps.cam1_voltage
    :field eps_cam1_current: icp_data.eps.cam1_current
    :field eps_cam2_voltage: icp_data.eps.cam2_voltage
    :field eps_cam2_current: icp_data.eps.cam2_current
    :field eps_hsom_voltage: icp_data.eps.hsom_voltage
    :field eps_hsom_current: icp_data.eps.hsom_current
    :field eps_cgp_5v_voltage: icp_data.eps.cgp_5v_voltage
    :field eps_cgp_mpb_voltage: icp_data.eps.cgp_mpb_voltage
    :field eps_cgp_current: icp_data.eps.cgp_current
    
    :field timestamp: icp_data.obc.common.unix_time
    :field obc_uptime: icp_data.obc.common.uptime
    :field obc_cpu_temperature: icp_data.obc.common.cpu_temperature
    :field obc_fmc_mram_temperature: icp_data.obc.fmc_mram_temperature
    :field obc_qspi_fram_temperature: icp_data.obc.qspi_fram_temperature
    :field obc_io_expander_temperature: icp_data.obc.io_expander_temperature
    
    :field aocs_gyroscope_x: icp_data.aocs.bmg160_gyroscope_x
    :field aocs_gyroscope_y: icp_data.aocs.bmg160_gyroscope_y
    :field aocs_gyroscope_z: icp_data.aocs.bmg160_gyroscope_z
    :field aocs_magnetometer_x: icp_data.aocs.lis3mdl_magnetometer_x
    :field aocs_magnetometer_y: icp_data.aocs.lis3mdl_magnetometer_y
    :field aocs_magnetometer_z: icp_data.aocs.lis3mdl_magnetometer_z
    :field aocs_sun_x_intensity1: icp_data.aocs.sun_x_intensity1
    :field aocs_sun_x_intensity2: icp_data.aocs.sun_x_intensity2
    :field aocs_sun_x_intensity3: icp_data.aocs.sun_x_intensity3
    :field aocs_sun_x_intensity4: icp_data.aocs.sun_x_intensity4
    :field aocs_sun_x_intensity5: icp_data.aocs.sun_x_intensity5
    :field aocs_sun_x_intensity6: icp_data.aocs.sun_x_intensity6
    :field aocs_sun_x_intensity_location1: icp_data.aocs.sun_x_intensity_location1
    :field aocs_sun_x_intensity_location2: icp_data.aocs.sun_x_intensity_location2
    :field aocs_sun_x_intensity_location3: icp_data.aocs.sun_x_intensity_location3
    :field aocs_sun_x_intensity_location4: icp_data.aocs.sun_x_intensity_location4
    :field aocs_sun_x_intensity_location5: icp_data.aocs.sun_x_intensity_location5
    :field aocs_sun_x_intensity_location6: icp_data.aocs.sun_x_intensity_location6
    :field aocs_sun_y_intensity1: icp_data.aocs.sun_y_intensity1
    :field aocs_sun_y_intensity2: icp_data.aocs.sun_y_intensity2
    :field aocs_sun_y_intensity3: icp_data.aocs.sun_y_intensity3
    :field aocs_sun_y_intensity4: icp_data.aocs.sun_y_intensity4
    :field aocs_sun_y_intensity5: icp_data.aocs.sun_y_intensity5
    :field aocs_sun_y_intensity6: icp_data.aocs.sun_y_intensity6
    :field aocs_sun_y_intensity_location1: icp_data.aocs.sun_y_intensity_location1
    :field aocs_sun_y_intensity_location2: icp_data.aocs.sun_y_intensity_location2
    :field aocs_sun_y_intensity_location3: icp_data.aocs.sun_y_intensity_location3
    :field aocs_sun_y_intensity_location4: icp_data.aocs.sun_y_intensity_location4
    :field aocs_sun_y_intensity_location5: icp_data.aocs.sun_y_intensity_location5
    :field aocs_sun_y_intensity_location6: icp_data.aocs.sun_y_intensity_location6
    :field aocs_temperature1: icp_data.aocs.mcp9808_temperature1
    :field aocs_temperature2: icp_data.aocs.mcp9808_temperature2
    :field aocs_mode_pointing: icp_data.aocs.pointing
    :field aocs_mode_detumbling: icp_data.aocs.detumbling
    :field aocs_mode_spin_up: icp_data.aocs.spin_up
    :field aocs_mode_diagnostics: icp_data.aocs.diagnostics
    :field aocs_mode_custom: icp_data.aocs.custom
    :field aocs_mode_low_precision: icp_data.aocs.low_precision
    :field aocs_mode_high_precision: icp_data.aocs.high_precision
    :field aocs_reaction_wheel1: icp_data.aocs.reaction_wheel1_rpm
    :field aocs_reaction_wheel2: icp_data.aocs.reaction_wheel2_rpm
    :field aocs_reaction_wheel3: icp_data.aocs.reaction_wheel3_rpm
    
    :field timestamp: icp_data.pcom.common.unix_time
    :field com_uptime: icp_data.pcom.common.uptime
    :field com_cpu_temperature: icp_data.pcom.common.cpu_temperature
    
    :field pcom_rssi: icp_data.pcom.rssi
    :field pcom_last_packet_time: icp_data.pcom.last_packet_time
    :field pcom_dropped_packets: icp_data.pcom.dropped_packets
    :field pcom_gs_packets: icp_data.pcom.gs_packets
    :field pcom_sent_packets: icp_data.pcom.sent_packets
    :field pcom_power_amplifier_temperature: icp_data.pcom.power_amplifier_temperature
    :field pcom_forward_rf_power: icp_data.pcom.forward_rf_power
    :field pcom_reflected_rf_power: icp_data.pcom.reflected_rf_power
    
    :field timestamp: icp_data.scom.common.unix_time
    :field scom_rssi: icp_data.scom.rssi
    :field scom_last_packet_time: icp_data.scom.last_packet_time
    :field scom_dropped_packets: icp_data.scom.dropped_packets
    :field scom_gs_packets: icp_data.scom.gs_packets
    :field scom_sent_packets: icp_data.scom.sent_packets
    :field scom_digipeated_packets: icp_data.scom.digipeated_packets
    :field scom_power_amplifier_temperature: icp_data.scom.power_amplifier_temperature
    :field scom_forward_rf_power: icp_data.scom.forward_rf_power
    :field scom_reflected_rf_power: icp_data.scom.reflected_rf_power
    
    :field timestamp: icp_data.sp_xplus.common.unix_time
    :field sp_xplus_uptime: icp_data.sp_xplus.common.uptime
    :field sp_xplus_cpu_temperature: icp_data.sp_xplus.common.cpu_temperature
    :field sp_xplus_temperature: icp_data.sp_xplus.temperature
    
    :field timestamp: icp_data.sp_xminus.common.unix_time
    :field sp_xminus_uptime: icp_data.sp_xminus.common.uptime
    :field sp_xminus_cpu_temperature: icp_data.sp_xminus.common.cpu_temperature
    :field sp_xminus_temperature: icp_data.sp_xminus.temperature
    :field sp_xminus_mppt_current: icp_data.sp_xminus.mppt_current
    :field sp_xminus_coil_current: icp_data.sp_xminus.coil_current
    
    :field timestamp: icp_data.sp_yplus.common.unix_time
    :field sp_yplus_uptime: icp_data.sp_yplus.common.uptime
    :field sp_yplus_cpu_temperature: icp_data.sp_yplus.common.cpu_temperature
    :field sp_yplus_temperature: icp_data.sp_yplus.temperature
    :field sp_yplus_mppt_current: icp_data.sp_yplus.mppt_current
    :field sp_yplus_coil_current: icp_data.sp_yplus.coil_current
    
    :field timestamp: icp_data.sp_yminus.common.unix_time
    :field sp_yminus_uptime: icp_data.sp_yminus.common.uptime
    :field sp_yminus_cpu_temperature: icp_data.sp_yminus.common.cpu_temperature
    :field sp_yminus_temperature: icp_data.sp_yminus.temperature
    :field sp_yminus_mppt_current: icp_data.sp_yminus.mppt_current
    
    :field timestamp: icp_data.sp_zplus.common.unix_time
    :field sp_zplus_uptime: icp_data.sp_zplus.common.uptime
    :field sp_zplus_cpu_temperature: icp_data.sp_zplus.common.cpu_temperature
    :field sp_zplus_temperature: icp_data.sp_zplus.temperature
    
    :field timestamp: icp_data.sp_zminus.common.unix_time
    :field sp_zminus_uptime: icp_data.sp_zminus.common.uptime
    :field sp_zminus_cpu_temperature: icp_data.sp_zminus.common.cpu_temperature
    :field sp_zminus_temperature: icp_data.sp_zminus.temperature
    :field sp_zminus_coil_current: icp_data.sp_zminus.coil_current
    
    :field timestamp: icp_data.st.common.unix_time
    :field st_uptime: icp_data.st.common.uptime
    :field st_cpu_temperature: icp_data.st.common.cpu_temperature
    :field st_fpga_temperature: icp_data.st.fpga_temperature
    :field st_sensor_temperature: icp_data.st.sensor_temperature
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_header = Estcube2.Ax25Header(self._io, self, self._root)
        self.icp_header = Estcube2.IcpHeader(self._io, self, self._root)
        self._raw_icp_data = self._io.read_bytes(self.icp_header.len)
        _io__raw_icp_data = KaitaiStream(BytesIO(self._raw_icp_data))
        self.icp_data = Estcube2.IcpData(_io__raw_icp_data, self, self._root)
        self.icp_crc = self._io.read_u2be()

    class IcpHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dst = self._io.read_u1()
            self.src = self._io.read_u1()
            self.len = self._io.read_u1()
            self.cmd = self._io.read_u1()
            self.uuid = self._io.read_bits_int_be(24)
            self._io.align_to_byte()
            self.mode = self._io.read_u1()


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Estcube2.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Estcube2.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Estcube2.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Estcube2.SsidMask(self._io, self, self._root)
            self.control = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class EpsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common = Estcube2.CommonHousekeeping(self._io, self, self._root)
            self.enable_reserved0 = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_adc1 = self._io.read_bits_int_be(1) != 0
            self.enable_adc2 = self._io.read_bits_int_be(1) != 0
            self.enable_adc3 = self._io.read_bits_int_be(1) != 0
            self.enable_fram1 = self._io.read_bits_int_be(1) != 0
            self.enable_fram2 = self._io.read_bits_int_be(1) != 0
            self.enable_rtc = self._io.read_bits_int_be(1) != 0
            self.enable_icp0 = self._io.read_bits_int_be(1) != 0
            self.enable_icp1 = self._io.read_bits_int_be(1) != 0
            self.enable_st = self._io.read_bits_int_be(1) != 0
            self.enable_obc = self._io.read_bits_int_be(1) != 0
            self.enable_com_3v3 = self._io.read_bits_int_be(1) != 0
            self.enable_com_dcdc = self._io.read_bits_int_be(1) != 0
            self.enable_com_pa = self._io.read_bits_int_be(1) != 0
            self.enable_eps_dcdc1 = self._io.read_bits_int_be(1) != 0
            self.enable_eps_dcdc2 = self._io.read_bits_int_be(1) != 0
            self.enable_sp = self._io.read_bits_int_be(1) != 0
            self.enable_cgp = self._io.read_bits_int_be(1) != 0
            self.enable_hscom = self._io.read_bits_int_be(1) != 0
            self.enable_eop1 = self._io.read_bits_int_be(1) != 0
            self.enable_eop2 = self._io.read_bits_int_be(1) != 0
            self.enable_esail = self._io.read_bits_int_be(1) != 0
            self.enable_obc_rw = self._io.read_bits_int_be(1) != 0
            self.enable_reserved25 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved26 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved27 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved28 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved29 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved30 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved31 = self._io.read_bits_int_be(1) != 0
            self.error_reserved0 = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_adc1 = self._io.read_bits_int_be(1) != 0
            self.error_adc2 = self._io.read_bits_int_be(1) != 0
            self.error_adc3 = self._io.read_bits_int_be(1) != 0
            self.error_fram1 = self._io.read_bits_int_be(1) != 0
            self.error_fram2 = self._io.read_bits_int_be(1) != 0
            self.error_rtc = self._io.read_bits_int_be(1) != 0
            self.error_icp0 = self._io.read_bits_int_be(1) != 0
            self.error_icp1 = self._io.read_bits_int_be(1) != 0
            self.error_st = self._io.read_bits_int_be(1) != 0
            self.error_dcdc1 = self._io.read_bits_int_be(1) != 0
            self.error_dcdc2 = self._io.read_bits_int_be(1) != 0
            self.error_chg1 = self._io.read_bits_int_be(1) != 0
            self.error_chg2 = self._io.read_bits_int_be(1) != 0
            self.error_dchg1 = self._io.read_bits_int_be(1) != 0
            self.error_dchg2 = self._io.read_bits_int_be(1) != 0
            self.error_reserved17 = self._io.read_bits_int_be(1) != 0
            self.error_reserved18 = self._io.read_bits_int_be(1) != 0
            self.error_reserved19 = self._io.read_bits_int_be(1) != 0
            self.error_reserved20 = self._io.read_bits_int_be(1) != 0
            self.error_reserved21 = self._io.read_bits_int_be(1) != 0
            self.error_reserved22 = self._io.read_bits_int_be(1) != 0
            self.error_reserved23 = self._io.read_bits_int_be(1) != 0
            self.error_reserved24 = self._io.read_bits_int_be(1) != 0
            self.error_reserved25 = self._io.read_bits_int_be(1) != 0
            self.error_reserved26 = self._io.read_bits_int_be(1) != 0
            self.error_reserved27 = self._io.read_bits_int_be(1) != 0
            self.error_reserved28 = self._io.read_bits_int_be(1) != 0
            self.error_reserved29 = self._io.read_bits_int_be(1) != 0
            self.error_reserved30 = self._io.read_bits_int_be(1) != 0
            self.error_reserved31 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.bus_voltage = self._io.read_u2le()
            self.avg_power_balance = self._io.read_s2le()
            self.battery_status_a = self._io.read_bits_int_be(1) != 0
            self.battery_status_b = self._io.read_bits_int_be(1) != 0
            self.battery_status_c = self._io.read_bits_int_be(1) != 0
            self.battery_status_d = self._io.read_bits_int_be(1) != 0
            self.battery_reserved4 = self._io.read_bits_int_be(1) != 0
            self.battery_reserved5 = self._io.read_bits_int_be(1) != 0
            self.battery_reserved6 = self._io.read_bits_int_be(1) != 0
            self.battery_reserved7 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.battery_current_a = self._io.read_u2le()
            self.battery_current_b = self._io.read_u2le()
            self.battery_voltage_a = self._io.read_u2le()
            self.battery_voltage_b = self._io.read_u2le()
            self.battery_voltage_c = self._io.read_u2le()
            self.battery_voltage_d = self._io.read_u2le()
            self.battery_temperature_a = self._io.read_u2le()
            self.battery_temperature_b = self._io.read_u2le()
            self.battery_temperature_c = self._io.read_u2le()
            self.battery_temperature_d = self._io.read_u2le()
            self.st_voltage = self._io.read_u2le()
            self.st_current = self._io.read_u2le()
            self.obc_voltage = self._io.read_u2le()
            self.obc_current = self._io.read_u2le()
            self.com_3v3_voltage = self._io.read_u2le()
            self.com_pa_voltage = self._io.read_u2le()
            self.com_3v3_current = self._io.read_u2le()
            self.com_pa_current = self._io.read_u2le()
            self.eps_voltage = self._io.read_u2le()
            self.eps_dcdc_3v3_voltage = self._io.read_u2le()
            self.eps_current = self._io.read_u2le()
            self.eps_dcdc1_current = self._io.read_u2le()
            self.eps_dcdc2_current = self._io.read_u2le()
            self.sp_voltage = self._io.read_u2le()
            self.sp_current = self._io.read_u2le()
            self.cpd_voltage = self._io.read_u2le()
            self.cpd_current = self._io.read_u2le()
            self.cam1_voltage = self._io.read_u2le()
            self.cam1_current = self._io.read_u2le()
            self.cam2_voltage = self._io.read_u2le()
            self.cam2_current = self._io.read_u2le()
            self.hsom_voltage = self._io.read_u2le()
            self.hsom_current = self._io.read_u2le()
            self.cgp_5v_voltage = self._io.read_u2le()
            self.cgp_mpb_voltage = self._io.read_u2le()
            self.cgp_current = self._io.read_u2le()


    class IcpData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if  ((self.src == 2) and (self.cmd == 131)) :
                self.pcom = Estcube2.PcomHousekeeping(self._io, self, self._root)

            if  ((self.src == 2) and (self.cmd == 131)) :
                self.scom = Estcube2.ScomHousekeeping(self._io, self, self._root)

            if  ((self.src == 3) and (self.cmd == 131)) :
                self.eps = Estcube2.EpsHousekeeping(self._io, self, self._root)

            if  ((self.src == 4) and (self.cmd == 131)) :
                self.obc = Estcube2.ObcHousekeeping(self._io, self, self._root)

            if  ((self.src == 4) and (self.cmd == 131)) :
                self.aocs = Estcube2.AocsHousekeeping(self._io, self, self._root)

            if  ((self.src == 5) and (self.cmd == 131)) :
                self.st = Estcube2.StarTrackerHousekeeping(self._io, self, self._root)

            if  ((self.src == 6) and (self.cmd == 131)) :
                self.sp_xplus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)

            if  ((self.src == 7) and (self.cmd == 131)) :
                self.sp_xminus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)

            if  ((self.src == 8) and (self.cmd == 131)) :
                self.sp_yplus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)

            if  ((self.src == 9) and (self.cmd == 131)) :
                self.sp_yminus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)

            if  ((self.src == 10) and (self.cmd == 131)) :
                self.sp_zplus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)

            if  ((self.src == 11) and (self.cmd == 131)) :
                self.sp_zminus = Estcube2.SidePanelHousekeeping(self._io, self, self._root)


        @property
        def src(self):
            if hasattr(self, '_m_src'):
                return self._m_src

            self._m_src = self._parent.icp_header.src
            return getattr(self, '_m_src', None)

        @property
        def cmd(self):
            if hasattr(self, '_m_cmd'):
                return self._m_cmd

            self._m_cmd = self._parent.icp_header.cmd
            return getattr(self, '_m_cmd', None)


    class AocsHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bmg160_gyroscope_x = self._io.read_s2le()
            self.bmg160_gyroscope_y = self._io.read_s2le()
            self.bmg160_gyroscope_z = self._io.read_s2le()
            self.lis3mdl_magnetometer_x = self._io.read_s2le()
            self.lis3mdl_magnetometer_y = self._io.read_s2le()
            self.lis3mdl_magnetometer_z = self._io.read_s2le()
            self.sun_x_intensity1 = self._io.read_u2le()
            self.sun_x_intensity2 = self._io.read_u2le()
            self.sun_x_intensity3 = self._io.read_u2le()
            self.sun_x_intensity4 = self._io.read_u2le()
            self.sun_x_intensity5 = self._io.read_u2le()
            self.sun_x_intensity6 = self._io.read_u2le()
            self.sun_x_intensity_location1 = self._io.read_u2le()
            self.sun_x_intensity_location2 = self._io.read_u2le()
            self.sun_x_intensity_location3 = self._io.read_u2le()
            self.sun_x_intensity_location4 = self._io.read_u2le()
            self.sun_x_intensity_location5 = self._io.read_u2le()
            self.sun_x_intensity_location6 = self._io.read_u2le()
            self.sun_y_intensity1 = self._io.read_u2le()
            self.sun_y_intensity2 = self._io.read_u2le()
            self.sun_y_intensity3 = self._io.read_u2le()
            self.sun_y_intensity4 = self._io.read_u2le()
            self.sun_y_intensity5 = self._io.read_u2le()
            self.sun_y_intensity6 = self._io.read_u2le()
            self.sun_y_intensity_location1 = self._io.read_u2le()
            self.sun_y_intensity_location2 = self._io.read_u2le()
            self.sun_y_intensity_location3 = self._io.read_u2le()
            self.sun_y_intensity_location4 = self._io.read_u2le()
            self.sun_y_intensity_location5 = self._io.read_u2le()
            self.sun_y_intensity_location6 = self._io.read_u2le()
            self.mcp9808_temperature1 = self._io.read_u1()
            self.mcp9808_temperature2 = self._io.read_u1()
            self.pointing = self._io.read_bits_int_be(1) != 0
            self.detumbling = self._io.read_bits_int_be(1) != 0
            self.spin_up = self._io.read_bits_int_be(1) != 0
            self.diagnostics = self._io.read_bits_int_be(1) != 0
            self.custom_mode = self._io.read_bits_int_be(1) != 0
            self.reserved = self._io.read_bits_int_be(1) != 0
            self.low_precision = self._io.read_bits_int_be(1) != 0
            self.high_precision = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.reaction_wheel1_rpm = self._io.read_s2le()
            self.reaction_wheel2_rpm = self._io.read_s2le()
            self.reaction_wheel3_rpm = self._io.read_s2le()


    class CommonHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.info_validity = self._io.read_bits_int_be(1) != 0
            self.hk_mode = self._io.read_bits_int_be(2)
            self.flags = self._io.read_bits_int_be(5)
            self._io.align_to_byte()
            self.unix_time = self._io.read_u4le()
            self.commands_queued = self._io.read_u1()
            self.commands_handled = self._io.read_u4le()
            self.commands_mcs = self._io.read_u2le()
            self.resets = self._io.read_u2le()
            self.last_reset_reason = self._io.read_u4le()
            self.uptime = self._io.read_u4le()
            self.errors = self._io.read_u4le()
            self.last_error_time = self._io.read_u4le()
            self.free_heap = self._io.read_u4le()
            self.active_tasks = self._io.read_u1()
            self.cpu_temperature = self._io.read_u1()
            self.current_firmware_slot = self._io.read_u1()
            self.slot1_firmware_version = self._io.read_u2le()
            self.slot2_firmware_version = self._io.read_u2le()
            self.slot3_firmware_version = self._io.read_u2le()
            self.slot4_firmware_version = self._io.read_u2le()
            self.slot1_crc_ok = self._io.read_bits_int_be(1) != 0
            self.slot2_crc_ok = self._io.read_bits_int_be(1) != 0
            self.slot3_crc_ok = self._io.read_bits_int_be(1) != 0
            self.slot4_crc_ok = self._io.read_bits_int_be(1) != 0
            self.reserved = self._io.read_bits_int_be(4)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class StarTrackerHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common = Estcube2.CommonHousekeeping(self._io, self, self._root)
            self.enable_reserved0 = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_camera = self._io.read_bits_int_be(1) != 0
            self.enable_fpga = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram = self._io.read_bits_int_be(1) != 0
            self.enable_spi_flash = self._io.read_bits_int_be(1) != 0
            self.enable_sdram = self._io.read_bits_int_be(1) != 0
            self.enable_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_reserved8 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved9 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved10 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved11 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved12 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved13 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved14 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved15 = self._io.read_bits_int_be(1) != 0
            self.error_mcu = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_camera = self._io.read_bits_int_be(1) != 0
            self.error_fpga = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram = self._io.read_bits_int_be(1) != 0
            self.error_spi_flash = self._io.read_bits_int_be(1) != 0
            self.error_sdram = self._io.read_bits_int_be(1) != 0
            self.error_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_reserved8 = self._io.read_bits_int_be(1) != 0
            self.error_reserved9 = self._io.read_bits_int_be(1) != 0
            self.error_reserved10 = self._io.read_bits_int_be(1) != 0
            self.error_reserved11 = self._io.read_bits_int_be(1) != 0
            self.error_reserved12 = self._io.read_bits_int_be(1) != 0
            self.error_reserved13 = self._io.read_bits_int_be(1) != 0
            self.error_reserved14 = self._io.read_bits_int_be(1) != 0
            self.error_reserved15 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.number_of_images_taken = self._io.read_u4le()
            self.number_of_successfull = self._io.read_u4le()
            self.number_of_fails = self._io.read_u4le()
            self.center_coordinate_x = self._io.read_f8le()
            self.center_coordinate_y = self._io.read_f8le()
            self.timestamp = self._io.read_u4le()
            self.database_version = self._io.read_u1()
            self.fpga_temperature = self._io.read_u2le()
            self.sensor_temperature = self._io.read_u2le()


    class SidePanelHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common = Estcube2.CommonHousekeeping(self._io, self, self._root)
            self.enable_reserved0 = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_sunsensor = self._io.read_bits_int_be(1) != 0
            self.enable_fram1 = self._io.read_bits_int_be(1) != 0
            self.enable_fram2 = self._io.read_bits_int_be(1) != 0
            self.enable_icp0 = self._io.read_bits_int_be(1) != 0
            self.enable_mag = self._io.read_bits_int_be(1) != 0
            self.enable_temp = self._io.read_bits_int_be(1) != 0
            self.enable_mppt = self._io.read_bits_int_be(1) != 0
            self.enable_coil = self._io.read_bits_int_be(1) != 0
            self.enable_reserved11 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved12 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved13 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved14 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved15 = self._io.read_bits_int_be(1) != 0
            self.error_mcu = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_sunsensor = self._io.read_bits_int_be(1) != 0
            self.error_fram1 = self._io.read_bits_int_be(1) != 0
            self.error_fram2 = self._io.read_bits_int_be(1) != 0
            self.error_icp0 = self._io.read_bits_int_be(1) != 0
            self.error_mag = self._io.read_bits_int_be(1) != 0
            self.error_temp = self._io.read_bits_int_be(1) != 0
            self.error_mppt = self._io.read_bits_int_be(1) != 0
            self.error_coil = self._io.read_bits_int_be(1) != 0
            self.error_reserved11 = self._io.read_bits_int_be(1) != 0
            self.error_reserved12 = self._io.read_bits_int_be(1) != 0
            self.error_reserved13 = self._io.read_bits_int_be(1) != 0
            self.error_reserved14 = self._io.read_bits_int_be(1) != 0
            self.error_reserved15 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.temperature = self._io.read_u1()
            self.mppt_current = self._io.read_u2le()
            self.coil_current = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Estcube2.Callsign(_io__raw_callsign_ror, self, self._root)


    class ObcHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common = Estcube2.CommonHousekeeping(self._io, self, self._root)
            self.enable_reserved = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_qspi_flash1 = self._io.read_bits_int_be(1) != 0
            self.enable_qspi_flash2 = self._io.read_bits_int_be(1) != 0
            self.enable_fmc_mram = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram1_obc = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram2_aocs1 = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram3_aocs2 = self._io.read_bits_int_be(1) != 0
            self.enable_io_expander = self._io.read_bits_int_be(1) != 0
            self.enable_fmc_mram_temp_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_qspi_flash_temp_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_io_expander_temp_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_rtc = self._io.read_bits_int_be(1) != 0
            self.enable_current_adc = self._io.read_bits_int_be(1) != 0
            self.enable_aocs1_gyroscope1 = self._io.read_bits_int_be(1) != 0
            self.enable_aocs1_gyroscope2 = self._io.read_bits_int_be(1) != 0
            self.enable_aocs1_magnetometer = self._io.read_bits_int_be(1) != 0
            self.enable_aocs1_accelerometer = self._io.read_bits_int_be(1) != 0
            self.enable_aocs1_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_aocs2_gyroscope1 = self._io.read_bits_int_be(1) != 0
            self.enable_aocs2_gyroscope2 = self._io.read_bits_int_be(1) != 0
            self.enable_aocs2_magnetometer = self._io.read_bits_int_be(1) != 0
            self.enable_aocs2_accelerometer = self._io.read_bits_int_be(1) != 0
            self.enable_aocs2_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.enable_payload_bus = self._io.read_bits_int_be(1) != 0
            self.enable_icp1_bus = self._io.read_bits_int_be(1) != 0
            self.enable_icp2_bus = self._io.read_bits_int_be(1) != 0
            self.enable_reaction_wheel1 = self._io.read_bits_int_be(1) != 0
            self.enable_reaction_wheel2 = self._io.read_bits_int_be(1) != 0
            self.enable_reaction_wheel3 = self._io.read_bits_int_be(1) != 0
            self.enable_oscillator = self._io.read_bits_int_be(1) != 0
            self.error_mcu = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_qspi_flash1 = self._io.read_bits_int_be(1) != 0
            self.error_qspi_flash2 = self._io.read_bits_int_be(1) != 0
            self.error_fmc_mram = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram1_obc = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram2_aocs1 = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram3_aocs2 = self._io.read_bits_int_be(1) != 0
            self.error_io_expander = self._io.read_bits_int_be(1) != 0
            self.error_mram_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_qspi_flash_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_io_expander_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_rtc = self._io.read_bits_int_be(1) != 0
            self.error_current_adc = self._io.read_bits_int_be(1) != 0
            self.error_aocs1_gyro1 = self._io.read_bits_int_be(1) != 0
            self.error_aocs1_gyro2 = self._io.read_bits_int_be(1) != 0
            self.error_aocs1_magnet = self._io.read_bits_int_be(1) != 0
            self.error_aocs1_accelerometer = self._io.read_bits_int_be(1) != 0
            self.error_aocs1_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_aocs2_gyro1 = self._io.read_bits_int_be(1) != 0
            self.error_aocs2_gyro2 = self._io.read_bits_int_be(1) != 0
            self.error_aocs2_magnet = self._io.read_bits_int_be(1) != 0
            self.error_aocs2_accelerometer = self._io.read_bits_int_be(1) != 0
            self.error_aocs2_temperature_sensor = self._io.read_bits_int_be(1) != 0
            self.error_payload_bus = self._io.read_bits_int_be(1) != 0
            self.error_icp1_bus = self._io.read_bits_int_be(1) != 0
            self.error_icp2_bus = self._io.read_bits_int_be(1) != 0
            self.error_reaction_wheel1 = self._io.read_bits_int_be(1) != 0
            self.error_reaction_wheel2 = self._io.read_bits_int_be(1) != 0
            self.error_reaction_wheel3 = self._io.read_bits_int_be(1) != 0
            self.error_oscillator = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.fmc_mram_temperature = self._io.read_u1()
            self.qspi_fram_temperature = self._io.read_u1()
            self.io_expander_temperature = self._io.read_u1()


    class PcomHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.common = Estcube2.CommonHousekeeping(self._io, self, self._root)
            self.enable_reserved0 = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_qspi_fram = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram = self._io.read_bits_int_be(1) != 0
            self.enable_transceiver = self._io.read_bits_int_be(1) != 0
            self.enable_dac = self._io.read_bits_int_be(1) != 0
            self.enable_icp0 = self._io.read_bits_int_be(1) != 0
            self.enable_icp1 = self._io.read_bits_int_be(1) != 0
            self.enable_icp2 = self._io.read_bits_int_be(1) != 0
            self.enable_oscillator = self._io.read_bits_int_be(1) != 0
            self.enable_temperature_sensor1 = self._io.read_bits_int_be(1) != 0
            self.enable_temperature_sensor2 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved2 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved3 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved4 = self._io.read_bits_int_be(1) != 0
            self.error_mcu = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_qspi_fram = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram = self._io.read_bits_int_be(1) != 0
            self.error_transceiver = self._io.read_bits_int_be(1) != 0
            self.error_dac = self._io.read_bits_int_be(1) != 0
            self.error_icp0 = self._io.read_bits_int_be(1) != 0
            self.error_icp1 = self._io.read_bits_int_be(1) != 0
            self.error_icp2 = self._io.read_bits_int_be(1) != 0
            self.error_oscillator = self._io.read_bits_int_be(1) != 0
            self.error_temperature_sensor1 = self._io.read_bits_int_be(1) != 0
            self.error_temperature_sensor2 = self._io.read_bits_int_be(1) != 0
            self.error_reserved1 = self._io.read_bits_int_be(1) != 0
            self.error_reserved2 = self._io.read_bits_int_be(1) != 0
            self.error_reserved3 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.rssi = self._io.read_u1()
            self.last_packet_time = self._io.read_bits_int_be(31)
            self.last_packet_time_invalid = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.dropped_packets = self._io.read_u4le()
            self.gs_packets = self._io.read_u4le()
            self.sent_packets = self._io.read_u4le()
            self.transceiver_gain = self._io.read_u1()
            self.power_amplifier_temperature = self._io.read_u1()
            self.forward_rf_power = self._io.read_s1()
            self.reflected_rf_power = self._io.read_s1()


    class ScomHousekeeping(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.enable_reserved1 = self._io.read_bits_int_be(1) != 0
            self.enable_internal_flash = self._io.read_bits_int_be(1) != 0
            self.enable_internal_sram = self._io.read_bits_int_be(1) != 0
            self.enable_qspi_fram = self._io.read_bits_int_be(1) != 0
            self.enable_spi_fram = self._io.read_bits_int_be(1) != 0
            self.enable_transceiver = self._io.read_bits_int_be(1) != 0
            self.enable_dac = self._io.read_bits_int_be(1) != 0
            self.enable_icp2 = self._io.read_bits_int_be(1) != 0
            self.enable_oscillator = self._io.read_bits_int_be(1) != 0
            self.enable_temperature_sensor1 = self._io.read_bits_int_be(1) != 0
            self.enable_temperature_sensor2 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved2 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved3 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved4 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved5 = self._io.read_bits_int_be(1) != 0
            self.enable_reserved6 = self._io.read_bits_int_be(1) != 0
            self.error_mcu = self._io.read_bits_int_be(1) != 0
            self.error_internal_flash = self._io.read_bits_int_be(1) != 0
            self.error_internal_sram = self._io.read_bits_int_be(1) != 0
            self.error_qspi_fram = self._io.read_bits_int_be(1) != 0
            self.error_spi_fram = self._io.read_bits_int_be(1) != 0
            self.error_transceiver = self._io.read_bits_int_be(1) != 0
            self.error_dac = self._io.read_bits_int_be(1) != 0
            self.error_icp2 = self._io.read_bits_int_be(1) != 0
            self.error_oscillator = self._io.read_bits_int_be(1) != 0
            self.error_temperature_sensor1 = self._io.read_bits_int_be(1) != 0
            self.error_temperature_sensor2 = self._io.read_bits_int_be(1) != 0
            self.error_reserved1 = self._io.read_bits_int_be(1) != 0
            self.error_reserved2 = self._io.read_bits_int_be(1) != 0
            self.error_reserved3 = self._io.read_bits_int_be(1) != 0
            self.error_reserved4 = self._io.read_bits_int_be(1) != 0
            self.error_reserved5 = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.rssi = self._io.read_u1()
            self.last_packet_time = self._io.read_bits_int_be(31)
            self.last_packet_time_invalid = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.dropped_packets = self._io.read_u4le()
            self.gs_packets = self._io.read_u4le()
            self.sent_packets = self._io.read_u4le()
            self.digipeated_packets = self._io.read_u4le()
            self.power_amplifier_temperature = self._io.read_u1()
            self.forward_rf_power = self._io.read_s1()
            self.reflected_rf_power = self._io.read_s1()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Co66(KaitaiStruct):
    """:field whole_beacon_ascii: mode.mode_check.whole_beacon_ascii
    :field beacon_mode: mode.mode_check.beacon_mode
    :field satellite_time: mode.mode_check.satellite_time
    :field batteries_voltage: mode.mode_check.batteries_voltage
    :field bus_voltage: mode.mode_check.bus_voltage
    :field solar_cell_1_current: mode.mode_check.solar_cell_1_current
    :field solar_cell_2_current: mode.mode_check.solar_cell_2_current
    :field solar_cell_3_current: mode.mode_check.solar_cell_3_current
    :field solar_cell_4_current: mode.mode_check.solar_cell_4_current
    :field solar_cell_5_current: mode.mode_check.solar_cell_5_current
    :field solar_cell_6_current: mode.mode_check.solar_cell_6_current
    :field battery_1_temperature: mode.mode_check.battery_1_temperature
    :field battery_2_temperature: mode.mode_check.battery_2_temperature
    :field transmitter_temperature: mode.mode_check.transmitter_temperature
    :field receiver_temperature: mode.mode_check.receiver_temperature
    :field cw_transmission_interval: mode.mode_check.cw_transmission_interval
    :field status_of_switch_1: mode.mode_check.status_of_switch_1
    :field status_of_switch_2: mode.mode_check.status_of_switch_2
    :field status_of_switch_3: mode.mode_check.status_of_switch_3
    :field mpu_reset_times_eps: mode.mode_check.mpu_reset_times_eps
    :field mpu_reset_times_fmr: mode.mode_check.mpu_reset_times_fmr
    :field mpu_reset_times_cdh: mode.mode_check.mpu_reset_times_cdh
    :field mpu_reset_times_cw: mode.mode_check.mpu_reset_times_cw
    :field cw_transmission_count: mode.mode_check.cw_transmission_count
    :field uplink_count: mode.mode_check.uplink_count
    :field command_status: mode.mode_check.command_status
    :field forced_no_charge_mode: mode.mode_check.forced_no_charge_mode
    :field mode_of_shunt_circuit: mode.mode_check.mode_of_shunt_circuit
    :field status_of_shunt_circuit: mode.mode_check.status_of_shunt_circuit
    :field address_block: mode.mode_check.address_block
    :field necessary_for_lengthcheck: mode.mode_check.necessary_for_lengthcheck
    
    .. seealso::
       Source - https://web.archive.org/web/20100215214032/http://cubesat.aero.cst.nihon-u.ac.jp/image/telemetry/CW_e/CW_Telemetry_Format_For_SEEDS_English.pdf
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.preamble = (self._io.read_bytes(5)).decode(u"ASCII")
        if not self.preamble == u"seeds":
            raise kaitaistruct.ValidationNotEqualError(u"seeds", self.preamble, self._io, u"/seq/0")
        self.mode = Co66.ModeT(self._io, self, self._root)

    class Four(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.whole_beacon_ascii = (self._io.read_bytes(72)).decode(u"utf-8")
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def beacon_mode(self):
            if hasattr(self, '_m_beacon_mode'):
                return self._m_beacon_mode

            self._m_beacon_mode = 4
            return getattr(self, '_m_beacon_mode', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.discard) != 0:
                self._m_necessary_for_lengthcheck = int(self.discard) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def solar_cell_3_current(self):
            if hasattr(self, '_m_solar_cell_3_current'):
                return self._m_solar_cell_3_current

            if  (((self.whole_beacon_ascii)[20:21] != u"*") and ((self.whole_beacon_ascii)[21:22] != u"*") and ((self.whole_beacon_ascii)[22:23] != u"*")) :
                self._m_solar_cell_3_current = (((5.0 * int((self.whole_beacon_ascii)[20:23], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_3_current', None)

        @property
        def solar_cell_5_current(self):
            if hasattr(self, '_m_solar_cell_5_current'):
                return self._m_solar_cell_5_current

            if  (((self.whole_beacon_ascii)[26:27] != u"*") and ((self.whole_beacon_ascii)[27:28] != u"*") and ((self.whole_beacon_ascii)[28:29] != u"*")) :
                self._m_solar_cell_5_current = (((5.0 * int((self.whole_beacon_ascii)[26:29], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_5_current', None)

        @property
        def command_status(self):
            if hasattr(self, '_m_command_status'):
                return self._m_command_status

            if  (((self.whole_beacon_ascii)[68:69] != u"*") and ((self.whole_beacon_ascii)[69:70] != u"*")) :
                self._m_command_status = int((self.whole_beacon_ascii)[68:70], 16)

            return getattr(self, '_m_command_status', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            if  (((self.whole_beacon_ascii)[0:1] != u"*") and ((self.whole_beacon_ascii)[1:2] != u"*") and ((self.whole_beacon_ascii)[2:3] != u"*") and ((self.whole_beacon_ascii)[3:4] != u"*") and ((self.whole_beacon_ascii)[4:5] != u"*") and ((self.whole_beacon_ascii)[5:6] != u"*") and ((self.whole_beacon_ascii)[6:7] != u"*") and ((self.whole_beacon_ascii)[7:8] != u"*")) :
                self._m_satellite_time = int((self.whole_beacon_ascii)[0:8], 16) // 2

            return getattr(self, '_m_satellite_time', None)

        @property
        def receiver_temperature(self):
            if hasattr(self, '_m_receiver_temperature'):
                return self._m_receiver_temperature

            if  (((self.whole_beacon_ascii)[41:42] != u"*") and ((self.whole_beacon_ascii)[42:43] != u"*") and ((self.whole_beacon_ascii)[43:44] != u"*")) :
                self._m_receiver_temperature = (((-0.062626 * (((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096))) - (38.305 * ((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096))) + 126.89)

            return getattr(self, '_m_receiver_temperature', None)

        @property
        def status_of_switch_2(self):
            if hasattr(self, '_m_status_of_switch_2'):
                return self._m_status_of_switch_2

            if (self.whole_beacon_ascii)[45:46] != u"*":
                self._m_status_of_switch_2 = ((int((self.whole_beacon_ascii)[45:46], 16) >> 1) & 1)

            return getattr(self, '_m_status_of_switch_2', None)

        @property
        def solar_cell_4_current(self):
            if hasattr(self, '_m_solar_cell_4_current'):
                return self._m_solar_cell_4_current

            if  (((self.whole_beacon_ascii)[23:24] != u"*") and ((self.whole_beacon_ascii)[24:25] != u"*") and ((self.whole_beacon_ascii)[25:26] != u"*")) :
                self._m_solar_cell_4_current = (((5.0 * int((self.whole_beacon_ascii)[23:26], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_4_current', None)

        @property
        def solar_cell_6_current(self):
            if hasattr(self, '_m_solar_cell_6_current'):
                return self._m_solar_cell_6_current

            if  (((self.whole_beacon_ascii)[29:30] != u"*") and ((self.whole_beacon_ascii)[30:31] != u"*") and ((self.whole_beacon_ascii)[31:32] != u"*")) :
                self._m_solar_cell_6_current = (((5.0 * int((self.whole_beacon_ascii)[29:32], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_6_current', None)

        @property
        def cw_transmission_count(self):
            if hasattr(self, '_m_cw_transmission_count'):
                return self._m_cw_transmission_count

            if  (((self.whole_beacon_ascii)[62:63] != u"*") and ((self.whole_beacon_ascii)[63:64] != u"*") and ((self.whole_beacon_ascii)[64:65] != u"*") and ((self.whole_beacon_ascii)[65:66] != u"*")) :
                self._m_cw_transmission_count = int((self.whole_beacon_ascii)[62:66], 16)

            return getattr(self, '_m_cw_transmission_count', None)

        @property
        def mode_of_shunt_circuit(self):
            if hasattr(self, '_m_mode_of_shunt_circuit'):
                return self._m_mode_of_shunt_circuit

            if (self.whole_beacon_ascii)[71:72] != u"*":
                self._m_mode_of_shunt_circuit = (int((self.whole_beacon_ascii)[71:72], 16) & 3)

            return getattr(self, '_m_mode_of_shunt_circuit', None)

        @property
        def mpu_reset_times_eps(self):
            if hasattr(self, '_m_mpu_reset_times_eps'):
                return self._m_mpu_reset_times_eps

            if  (((self.whole_beacon_ascii)[46:47] != u"*") and ((self.whole_beacon_ascii)[47:48] != u"*") and ((self.whole_beacon_ascii)[48:49] != u"*") and ((self.whole_beacon_ascii)[49:50] != u"*")) :
                self._m_mpu_reset_times_eps = int((self.whole_beacon_ascii)[46:50], 16)

            return getattr(self, '_m_mpu_reset_times_eps', None)

        @property
        def uplink_count(self):
            if hasattr(self, '_m_uplink_count'):
                return self._m_uplink_count

            if  (((self.whole_beacon_ascii)[66:67] != u"*") and ((self.whole_beacon_ascii)[67:68] != u"*")) :
                self._m_uplink_count = int((self.whole_beacon_ascii)[66:68], 16)

            return getattr(self, '_m_uplink_count', None)

        @property
        def forced_no_charge_mode(self):
            if hasattr(self, '_m_forced_no_charge_mode'):
                return self._m_forced_no_charge_mode

            if (self.whole_beacon_ascii)[70:71] != u"*":
                self._m_forced_no_charge_mode = (int((self.whole_beacon_ascii)[70:71], 16) >> 3)

            return getattr(self, '_m_forced_no_charge_mode', None)

        @property
        def cw_transmission_interval(self):
            if hasattr(self, '_m_cw_transmission_interval'):
                return self._m_cw_transmission_interval

            if (self.whole_beacon_ascii)[44:45] != u"*":
                self._m_cw_transmission_interval = (int((self.whole_beacon_ascii)[44:45], 16) * 3)

            return getattr(self, '_m_cw_transmission_interval', None)

        @property
        def batteries_voltage(self):
            if hasattr(self, '_m_batteries_voltage'):
                return self._m_batteries_voltage

            if  (((self.whole_beacon_ascii)[8:9] != u"*") and ((self.whole_beacon_ascii)[9:10] != u"*") and ((self.whole_beacon_ascii)[10:11] != u"*")) :
                self._m_batteries_voltage = ((5.0 * int((self.whole_beacon_ascii)[8:11], 16)) / 4096)

            return getattr(self, '_m_batteries_voltage', None)

        @property
        def transmitter_temperature(self):
            if hasattr(self, '_m_transmitter_temperature'):
                return self._m_transmitter_temperature

            if  (((self.whole_beacon_ascii)[38:39] != u"*") and ((self.whole_beacon_ascii)[39:40] != u"*") and ((self.whole_beacon_ascii)[40:41] != u"*")) :
                self._m_transmitter_temperature = (((-0.38082 * (((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096))) - (36.125 * ((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096))) + 121.31)

            return getattr(self, '_m_transmitter_temperature', None)

        @property
        def mpu_reset_times_cdh(self):
            if hasattr(self, '_m_mpu_reset_times_cdh'):
                return self._m_mpu_reset_times_cdh

            if  (((self.whole_beacon_ascii)[54:55] != u"*") and ((self.whole_beacon_ascii)[55:56] != u"*") and ((self.whole_beacon_ascii)[56:57] != u"*") and ((self.whole_beacon_ascii)[57:58] != u"*")) :
                self._m_mpu_reset_times_cdh = int((self.whole_beacon_ascii)[54:58], 16)

            return getattr(self, '_m_mpu_reset_times_cdh', None)

        @property
        def mpu_reset_times_fmr(self):
            if hasattr(self, '_m_mpu_reset_times_fmr'):
                return self._m_mpu_reset_times_fmr

            if  (((self.whole_beacon_ascii)[50:51] != u"*") and ((self.whole_beacon_ascii)[51:52] != u"*") and ((self.whole_beacon_ascii)[52:53] != u"*") and ((self.whole_beacon_ascii)[53:54] != u"*")) :
                self._m_mpu_reset_times_fmr = int((self.whole_beacon_ascii)[50:54], 16)

            return getattr(self, '_m_mpu_reset_times_fmr', None)

        @property
        def status_of_switch_3(self):
            if hasattr(self, '_m_status_of_switch_3'):
                return self._m_status_of_switch_3

            if (self.whole_beacon_ascii)[45:46] != u"*":
                self._m_status_of_switch_3 = ((int((self.whole_beacon_ascii)[45:46], 16) >> 2) & 1)

            return getattr(self, '_m_status_of_switch_3', None)

        @property
        def mpu_reset_times_cw(self):
            if hasattr(self, '_m_mpu_reset_times_cw'):
                return self._m_mpu_reset_times_cw

            if  (((self.whole_beacon_ascii)[58:59] != u"*") and ((self.whole_beacon_ascii)[59:60] != u"*") and ((self.whole_beacon_ascii)[60:61] != u"*") and ((self.whole_beacon_ascii)[61:62] != u"*")) :
                self._m_mpu_reset_times_cw = int((self.whole_beacon_ascii)[58:62], 16)

            return getattr(self, '_m_mpu_reset_times_cw', None)

        @property
        def solar_cell_1_current(self):
            if hasattr(self, '_m_solar_cell_1_current'):
                return self._m_solar_cell_1_current

            if  (((self.whole_beacon_ascii)[14:15] != u"*") and ((self.whole_beacon_ascii)[15:16] != u"*") and ((self.whole_beacon_ascii)[16:17] != u"*")) :
                self._m_solar_cell_1_current = (((5.0 * int((self.whole_beacon_ascii)[14:17], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_1_current', None)

        @property
        def status_of_shunt_circuit(self):
            if hasattr(self, '_m_status_of_shunt_circuit'):
                return self._m_status_of_shunt_circuit

            if (self.whole_beacon_ascii)[71:72] != u"*":
                self._m_status_of_shunt_circuit = ((int((self.whole_beacon_ascii)[71:72], 16) >> 2) & 1)

            return getattr(self, '_m_status_of_shunt_circuit', None)

        @property
        def bus_voltage(self):
            if hasattr(self, '_m_bus_voltage'):
                return self._m_bus_voltage

            if  (((self.whole_beacon_ascii)[11:12] != u"*") and ((self.whole_beacon_ascii)[12:13] != u"*") and ((self.whole_beacon_ascii)[13:14] != u"*")) :
                self._m_bus_voltage = ((5.0 * int((self.whole_beacon_ascii)[11:14], 16)) / 4096)

            return getattr(self, '_m_bus_voltage', None)

        @property
        def battery_1_temperature(self):
            if hasattr(self, '_m_battery_1_temperature'):
                return self._m_battery_1_temperature

            if  (((self.whole_beacon_ascii)[32:33] != u"*") and ((self.whole_beacon_ascii)[33:34] != u"*") and ((self.whole_beacon_ascii)[34:35] != u"*")) :
                self._m_battery_1_temperature = (((0.15797 * (((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096))) - (39.553 * ((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096))) + 129.59)

            return getattr(self, '_m_battery_1_temperature', None)

        @property
        def battery_2_temperature(self):
            if hasattr(self, '_m_battery_2_temperature'):
                return self._m_battery_2_temperature

            if  (((self.whole_beacon_ascii)[35:36] != u"*") and ((self.whole_beacon_ascii)[36:37] != u"*") and ((self.whole_beacon_ascii)[37:38] != u"*")) :
                self._m_battery_2_temperature = (((0.18923 * (((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096))) - (39.27 * ((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096))) + 128.33)

            return getattr(self, '_m_battery_2_temperature', None)

        @property
        def status_of_switch_1(self):
            if hasattr(self, '_m_status_of_switch_1'):
                return self._m_status_of_switch_1

            if (self.whole_beacon_ascii)[45:46] != u"*":
                self._m_status_of_switch_1 = (int((self.whole_beacon_ascii)[45:46], 16) & 1)

            return getattr(self, '_m_status_of_switch_1', None)

        @property
        def solar_cell_2_current(self):
            if hasattr(self, '_m_solar_cell_2_current'):
                return self._m_solar_cell_2_current

            if  (((self.whole_beacon_ascii)[17:18] != u"*") and ((self.whole_beacon_ascii)[18:19] != u"*") and ((self.whole_beacon_ascii)[19:20] != u"*")) :
                self._m_solar_cell_2_current = (((5.0 * int((self.whole_beacon_ascii)[17:20], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_2_current', None)


    class Three(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.whole_beacon_ascii = (self._io.read_bytes(48)).decode(u"utf-8")
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def beacon_mode(self):
            if hasattr(self, '_m_beacon_mode'):
                return self._m_beacon_mode

            self._m_beacon_mode = 3
            return getattr(self, '_m_beacon_mode', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.discard) != 0:
                self._m_necessary_for_lengthcheck = int(self.discard) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def solar_cell_3_current(self):
            if hasattr(self, '_m_solar_cell_3_current'):
                return self._m_solar_cell_3_current

            if  (((self.whole_beacon_ascii)[18:19] != u"*") and ((self.whole_beacon_ascii)[19:20] != u"*") and ((self.whole_beacon_ascii)[20:21] != u"*")) :
                self._m_solar_cell_3_current = (((5.0 * int((self.whole_beacon_ascii)[18:21], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_3_current', None)

        @property
        def solar_cell_5_current(self):
            if hasattr(self, '_m_solar_cell_5_current'):
                return self._m_solar_cell_5_current

            if  (((self.whole_beacon_ascii)[24:25] != u"*") and ((self.whole_beacon_ascii)[25:26] != u"*") and ((self.whole_beacon_ascii)[26:27] != u"*")) :
                self._m_solar_cell_5_current = (((5.0 * int((self.whole_beacon_ascii)[24:27], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_5_current', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            if  (((self.whole_beacon_ascii)[0:1] != u"*") and ((self.whole_beacon_ascii)[1:2] != u"*") and ((self.whole_beacon_ascii)[2:3] != u"*") and ((self.whole_beacon_ascii)[3:4] != u"*") and ((self.whole_beacon_ascii)[4:5] != u"*") and ((self.whole_beacon_ascii)[5:6] != u"*") and ((self.whole_beacon_ascii)[6:7] != u"*") and ((self.whole_beacon_ascii)[7:8] != u"*")) :
                self._m_satellite_time = int((self.whole_beacon_ascii)[0:8], 16) // 2

            return getattr(self, '_m_satellite_time', None)

        @property
        def receiver_temperature(self):
            if hasattr(self, '_m_receiver_temperature'):
                return self._m_receiver_temperature

            if  (((self.whole_beacon_ascii)[39:40] != u"*") and ((self.whole_beacon_ascii)[40:41] != u"*") and ((self.whole_beacon_ascii)[41:42] != u"*")) :
                self._m_receiver_temperature = (((-0.062626 * (((5.0 * int((self.whole_beacon_ascii)[39:42], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[39:42], 16)) / 4096))) - (38.305 * ((5.0 * int((self.whole_beacon_ascii)[39:42], 16)) / 4096))) + 126.89)

            return getattr(self, '_m_receiver_temperature', None)

        @property
        def solar_cell_4_current(self):
            if hasattr(self, '_m_solar_cell_4_current'):
                return self._m_solar_cell_4_current

            if  (((self.whole_beacon_ascii)[21:22] != u"*") and ((self.whole_beacon_ascii)[22:23] != u"*") and ((self.whole_beacon_ascii)[23:24] != u"*")) :
                self._m_solar_cell_4_current = (((5.0 * int((self.whole_beacon_ascii)[21:24], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_4_current', None)

        @property
        def solar_cell_6_current(self):
            if hasattr(self, '_m_solar_cell_6_current'):
                return self._m_solar_cell_6_current

            if  (((self.whole_beacon_ascii)[27:28] != u"*") and ((self.whole_beacon_ascii)[28:29] != u"*") and ((self.whole_beacon_ascii)[29:30] != u"*")) :
                self._m_solar_cell_6_current = (((5.0 * int((self.whole_beacon_ascii)[27:30], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_6_current', None)

        @property
        def batteries_voltage(self):
            if hasattr(self, '_m_batteries_voltage'):
                return self._m_batteries_voltage

            if  (((self.whole_beacon_ascii)[42:43] != u"*") and ((self.whole_beacon_ascii)[43:44] != u"*") and ((self.whole_beacon_ascii)[44:45] != u"*")) :
                self._m_batteries_voltage = ((5.0 * int((self.whole_beacon_ascii)[42:45], 16)) / 4096)

            return getattr(self, '_m_batteries_voltage', None)

        @property
        def transmitter_temperature(self):
            if hasattr(self, '_m_transmitter_temperature'):
                return self._m_transmitter_temperature

            if  (((self.whole_beacon_ascii)[36:37] != u"*") and ((self.whole_beacon_ascii)[37:38] != u"*") and ((self.whole_beacon_ascii)[38:39] != u"*")) :
                self._m_transmitter_temperature = (((-0.38082 * (((5.0 * int((self.whole_beacon_ascii)[36:39], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[36:39], 16)) / 4096))) - (36.125 * ((5.0 * int((self.whole_beacon_ascii)[36:39], 16)) / 4096))) + 121.31)

            return getattr(self, '_m_transmitter_temperature', None)

        @property
        def solar_cell_1_current(self):
            if hasattr(self, '_m_solar_cell_1_current'):
                return self._m_solar_cell_1_current

            if  (((self.whole_beacon_ascii)[12:13] != u"*") and ((self.whole_beacon_ascii)[13:14] != u"*") and ((self.whole_beacon_ascii)[14:115] != u"*")) :
                self._m_solar_cell_1_current = (((5.0 * int((self.whole_beacon_ascii)[12:15], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_1_current', None)

        @property
        def bus_voltage(self):
            if hasattr(self, '_m_bus_voltage'):
                return self._m_bus_voltage

            if  (((self.whole_beacon_ascii)[45:46] != u"*") and ((self.whole_beacon_ascii)[46:47] != u"*") and ((self.whole_beacon_ascii)[47:48] != u"*")) :
                self._m_bus_voltage = ((5.0 * int((self.whole_beacon_ascii)[45:48], 16)) / 4096)

            return getattr(self, '_m_bus_voltage', None)

        @property
        def battery_1_temperature(self):
            if hasattr(self, '_m_battery_1_temperature'):
                return self._m_battery_1_temperature

            if  (((self.whole_beacon_ascii)[30:31] != u"*") and ((self.whole_beacon_ascii)[31:32] != u"*") and ((self.whole_beacon_ascii)[32:33] != u"*")) :
                self._m_battery_1_temperature = (((0.15797 * (((5.0 * int((self.whole_beacon_ascii)[30:33], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[30:33], 16)) / 4096))) - (39.553 * ((5.0 * int((self.whole_beacon_ascii)[30:33], 16)) / 4096))) + 129.59)

            return getattr(self, '_m_battery_1_temperature', None)

        @property
        def address_block(self):
            if hasattr(self, '_m_address_block'):
                return self._m_address_block

            if  (((self.whole_beacon_ascii)[8:9] != u"*") and ((self.whole_beacon_ascii)[9:10] != u"*") and ((self.whole_beacon_ascii)[10:11] != u"*") and ((self.whole_beacon_ascii)[11:12] != u"*")) :
                self._m_address_block = int((self.whole_beacon_ascii)[8:12], 16)

            return getattr(self, '_m_address_block', None)

        @property
        def battery_2_temperature(self):
            if hasattr(self, '_m_battery_2_temperature'):
                return self._m_battery_2_temperature

            if  (((self.whole_beacon_ascii)[33:34] != u"*") and ((self.whole_beacon_ascii)[34:35] != u"*") and ((self.whole_beacon_ascii)[35:36] != u"*")) :
                self._m_battery_2_temperature = (((0.18923 * (((5.0 * int((self.whole_beacon_ascii)[33:36], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[33:36], 16)) / 4096))) - (39.27 * ((5.0 * int((self.whole_beacon_ascii)[33:36], 16)) / 4096))) + 128.33)

            return getattr(self, '_m_battery_2_temperature', None)

        @property
        def solar_cell_2_current(self):
            if hasattr(self, '_m_solar_cell_2_current'):
                return self._m_solar_cell_2_current

            if  (((self.whole_beacon_ascii)[15:16] != u"*") and ((self.whole_beacon_ascii)[16:17] != u"*") and ((self.whole_beacon_ascii)[17:18] != u"*")) :
                self._m_solar_cell_2_current = (((5.0 * int((self.whole_beacon_ascii)[15:18], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_2_current', None)


    class Zero(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.whole_beacon_ascii = (self._io.read_bytes(6)).decode(u"utf-8")
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def beacon_mode(self):
            if hasattr(self, '_m_beacon_mode'):
                return self._m_beacon_mode

            self._m_beacon_mode = 0
            return getattr(self, '_m_beacon_mode', None)

        @property
        def batteries_voltage(self):
            if hasattr(self, '_m_batteries_voltage'):
                return self._m_batteries_voltage

            if  (((self.whole_beacon_ascii)[0:1] != u"*") and ((self.whole_beacon_ascii)[1:2] != u"*") and ((self.whole_beacon_ascii)[2:3] != u"*")) :
                self._m_batteries_voltage = ((5.0 * int((self.whole_beacon_ascii)[0:3], 16)) / 4096)

            return getattr(self, '_m_batteries_voltage', None)

        @property
        def bus_voltage(self):
            if hasattr(self, '_m_bus_voltage'):
                return self._m_bus_voltage

            if  (((self.whole_beacon_ascii)[3:4] != u"*") and ((self.whole_beacon_ascii)[4:5] != u"*") and ((self.whole_beacon_ascii)[5:6] != u"*")) :
                self._m_bus_voltage = ((5.0 * int((self.whole_beacon_ascii)[3:6], 16)) / 4096)

            return getattr(self, '_m_bus_voltage', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.discard) != 0:
                self._m_necessary_for_lengthcheck = int(self.discard) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)


    class Discard(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.discard = self._io.read_bits_int_be(1) != 0

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            self._m_necessary_for_lengthcheck = int(self.discard) // 0
            return getattr(self, '_m_necessary_for_lengthcheck', None)


    class Six(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.whole_beacon_ascii = (self._io.read_bytes(3)).decode(u"utf-8")
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def beacon_mode(self):
            if hasattr(self, '_m_beacon_mode'):
                return self._m_beacon_mode

            self._m_beacon_mode = 6
            return getattr(self, '_m_beacon_mode', None)

        @property
        def batteries_voltage(self):
            if hasattr(self, '_m_batteries_voltage'):
                return self._m_batteries_voltage

            if  (((self.whole_beacon_ascii)[0:1] != u"*") and ((self.whole_beacon_ascii)[1:2] != u"*") and ((self.whole_beacon_ascii)[2:3] != u"*")) :
                self._m_batteries_voltage = ((5.0 * int((self.whole_beacon_ascii)[0:3], 16)) / 4096)

            return getattr(self, '_m_batteries_voltage', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.discard) != 0:
                self._m_necessary_for_lengthcheck = int(self.discard) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)


    class ModeT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 52:
                self.mode_check = Co66.Four(self._io, self, self._root)
            elif _on == 51:
                self.mode_check = Co66.Three(self._io, self, self._root)
            elif _on == 48:
                self.mode_check = Co66.Zero(self._io, self, self._root)
            elif _on == 49:
                self.mode_check = Co66.One(self._io, self, self._root)
            elif _on == 54:
                self.mode_check = Co66.Six(self._io, self, self._root)
            else:
                self.mode_check = Co66.Discard(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            self._m_check = self._io.read_u1()
            return getattr(self, '_m_check', None)


    class One(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.whole_beacon_ascii = (self._io.read_bytes(45)).decode(u"utf-8")
            self.discard = (self._io.read_bytes_full()).decode(u"utf-8")

        @property
        def beacon_mode(self):
            if hasattr(self, '_m_beacon_mode'):
                return self._m_beacon_mode

            self._m_beacon_mode = 1
            return getattr(self, '_m_beacon_mode', None)

        @property
        def necessary_for_lengthcheck(self):
            if hasattr(self, '_m_necessary_for_lengthcheck'):
                return self._m_necessary_for_lengthcheck

            if len(self.discard) != 0:
                self._m_necessary_for_lengthcheck = int(self.discard) // 0

            return getattr(self, '_m_necessary_for_lengthcheck', None)

        @property
        def solar_cell_3_current(self):
            if hasattr(self, '_m_solar_cell_3_current'):
                return self._m_solar_cell_3_current

            if  (((self.whole_beacon_ascii)[20:21] != u"*") and ((self.whole_beacon_ascii)[21:22] != u"*") and ((self.whole_beacon_ascii)[22:23] != u"*")) :
                self._m_solar_cell_3_current = (((5.0 * int((self.whole_beacon_ascii)[20:23], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_3_current', None)

        @property
        def solar_cell_5_current(self):
            if hasattr(self, '_m_solar_cell_5_current'):
                return self._m_solar_cell_5_current

            if  (((self.whole_beacon_ascii)[26:27] != u"*") and ((self.whole_beacon_ascii)[27:28] != u"*") and ((self.whole_beacon_ascii)[28:29] != u"*")) :
                self._m_solar_cell_5_current = (((5.0 * int((self.whole_beacon_ascii)[26:29], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_5_current', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            if  (((self.whole_beacon_ascii)[0:1] != u"*") and ((self.whole_beacon_ascii)[1:2] != u"*") and ((self.whole_beacon_ascii)[2:3] != u"*") and ((self.whole_beacon_ascii)[3:4] != u"*") and ((self.whole_beacon_ascii)[4:5] != u"*") and ((self.whole_beacon_ascii)[5:6] != u"*") and ((self.whole_beacon_ascii)[6:7] != u"*") and ((self.whole_beacon_ascii)[7:8] != u"*")) :
                self._m_satellite_time = int((self.whole_beacon_ascii)[0:8], 16) // 2

            return getattr(self, '_m_satellite_time', None)

        @property
        def receiver_temperature(self):
            if hasattr(self, '_m_receiver_temperature'):
                return self._m_receiver_temperature

            if  (((self.whole_beacon_ascii)[41:42] != u"*") and ((self.whole_beacon_ascii)[42:43] != u"*") and ((self.whole_beacon_ascii)[43:44] != u"*")) :
                self._m_receiver_temperature = (((-0.062626 * (((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096))) - (38.305 * ((5.0 * int((self.whole_beacon_ascii)[41:44], 16)) / 4096))) + 126.89)

            return getattr(self, '_m_receiver_temperature', None)

        @property
        def solar_cell_4_current(self):
            if hasattr(self, '_m_solar_cell_4_current'):
                return self._m_solar_cell_4_current

            if  (((self.whole_beacon_ascii)[23:24] != u"*") and ((self.whole_beacon_ascii)[24:25] != u"*") and ((self.whole_beacon_ascii)[25:26] != u"*")) :
                self._m_solar_cell_4_current = (((5.0 * int((self.whole_beacon_ascii)[23:26], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_4_current', None)

        @property
        def solar_cell_6_current(self):
            if hasattr(self, '_m_solar_cell_6_current'):
                return self._m_solar_cell_6_current

            if  (((self.whole_beacon_ascii)[29:30] != u"*") and ((self.whole_beacon_ascii)[30:31] != u"*") and ((self.whole_beacon_ascii)[31:32] != u"*")) :
                self._m_solar_cell_6_current = (((5.0 * int((self.whole_beacon_ascii)[29:32], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_6_current', None)

        @property
        def cw_transmission_interval(self):
            if hasattr(self, '_m_cw_transmission_interval'):
                return self._m_cw_transmission_interval

            if (self.whole_beacon_ascii)[44:45] != u"*":
                self._m_cw_transmission_interval = (int((self.whole_beacon_ascii)[44:45], 16) * 3)

            return getattr(self, '_m_cw_transmission_interval', None)

        @property
        def batteries_voltage(self):
            if hasattr(self, '_m_batteries_voltage'):
                return self._m_batteries_voltage

            if  (((self.whole_beacon_ascii)[8:9] != u"*") and ((self.whole_beacon_ascii)[9:10] != u"*") and ((self.whole_beacon_ascii)[10:11] != u"*")) :
                self._m_batteries_voltage = ((5.0 * int((self.whole_beacon_ascii)[8:11], 16)) / 4096)

            return getattr(self, '_m_batteries_voltage', None)

        @property
        def transmitter_temperature(self):
            if hasattr(self, '_m_transmitter_temperature'):
                return self._m_transmitter_temperature

            if  (((self.whole_beacon_ascii)[38:39] != u"*") and ((self.whole_beacon_ascii)[39:40] != u"*") and ((self.whole_beacon_ascii)[40:41] != u"*")) :
                self._m_transmitter_temperature = (((-0.38082 * (((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096))) - (36.125 * ((5.0 * int((self.whole_beacon_ascii)[38:41], 16)) / 4096))) + 121.31)

            return getattr(self, '_m_transmitter_temperature', None)

        @property
        def solar_cell_1_current(self):
            if hasattr(self, '_m_solar_cell_1_current'):
                return self._m_solar_cell_1_current

            if  (((self.whole_beacon_ascii)[14:15] != u"*") and ((self.whole_beacon_ascii)[15:16] != u"*") and ((self.whole_beacon_ascii)[16:17] != u"*")) :
                self._m_solar_cell_1_current = (((5.0 * int((self.whole_beacon_ascii)[14:17], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_1_current', None)

        @property
        def bus_voltage(self):
            if hasattr(self, '_m_bus_voltage'):
                return self._m_bus_voltage

            if  (((self.whole_beacon_ascii)[11:12] != u"*") and ((self.whole_beacon_ascii)[12:13] != u"*") and ((self.whole_beacon_ascii)[13:14] != u"*")) :
                self._m_bus_voltage = ((5.0 * int((self.whole_beacon_ascii)[11:14], 16)) / 4096)

            return getattr(self, '_m_bus_voltage', None)

        @property
        def battery_1_temperature(self):
            if hasattr(self, '_m_battery_1_temperature'):
                return self._m_battery_1_temperature

            if  (((self.whole_beacon_ascii)[32:33] != u"*") and ((self.whole_beacon_ascii)[33:34] != u"*") and ((self.whole_beacon_ascii)[34:35] != u"*")) :
                self._m_battery_1_temperature = (((0.15797 * (((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096))) - (39.553 * ((5.0 * int((self.whole_beacon_ascii)[32:35], 16)) / 4096))) + 129.59)

            return getattr(self, '_m_battery_1_temperature', None)

        @property
        def battery_2_temperature(self):
            if hasattr(self, '_m_battery_2_temperature'):
                return self._m_battery_2_temperature

            if  (((self.whole_beacon_ascii)[35:36] != u"*") and ((self.whole_beacon_ascii)[36:37] != u"*") and ((self.whole_beacon_ascii)[37:38] != u"*")) :
                self._m_battery_2_temperature = (((0.18923 * (((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096) * ((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096))) - (39.27 * ((5.0 * int((self.whole_beacon_ascii)[35:38], 16)) / 4096))) + 128.33)

            return getattr(self, '_m_battery_2_temperature', None)

        @property
        def solar_cell_2_current(self):
            if hasattr(self, '_m_solar_cell_2_current'):
                return self._m_solar_cell_2_current

            if  (((self.whole_beacon_ascii)[17:18] != u"*") and ((self.whole_beacon_ascii)[18:19] != u"*") and ((self.whole_beacon_ascii)[19:20] != u"*")) :
                self._m_solar_cell_2_current = (((5.0 * int((self.whole_beacon_ascii)[17:20], 16)) / 4096) * 90.90909)

            return getattr(self, '_m_solar_cell_2_current', None)






"""
some commonly used methods
"""

import json
import sys

PY2 = sys.version_info < (3, 0)


def check_type(result):
    """
    Check return types of decoder
    """

    count = 0
    for value in result:
        count = count + 1
        if PY2:
            assert isinstance(result[value], (unicode, str, int, float))  # noqa: F821,E501 pylint: disable=E0602
        else:
            assert isinstance(result[value], (str, int, float))
    return count


def load_objects(decoder, dut, frame):
    """
    Decode frame and load result into JSON objects
    """

    result = decoder.get_fields(getattr(decoder, dut).from_bytes(frame))
    result = json.dumps(result)
    result = json.loads(result)
    return result



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Innosat16(KaitaiStruct):
    """:field callsign: data.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: data.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: data.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: data.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: data.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: data.ax25_header.src_ssid_raw.ssid
    :field ctl: data.ax25_header.ctl
    :field pid: data.ax25_header.pid
    :field beacon_id: data.payload.beacon_id
    :field eps_1_mode: data.payload.eps_1_mode
    :field eps_1_consumption_current: data.payload.eps_1_consumption_current
    :field eps_1_solar_cells_current: data.payload.eps_1_solar_cells_current
    :field eps_1_cell_voltage_full: data.payload.eps_1_cell_voltage_full
    :field eps_1_battery_temperature: data.payload.eps_1_battery_temperature
    :field eps_1_temperature_sp_y_pos: data.payload.eps_1_temperature_sp_y_pos
    :field eps_1_temperature_sp_y_neg: data.payload.eps_1_temperature_sp_y_neg
    :field eps_1_temperature_sp_x_pos: data.payload.eps_1_temperature_sp_x_pos
    :field eps_1_temperature_sp_x_neg: data.payload.eps_1_temperature_sp_x_neg
    :field eps_1_systems_status: data.payload.eps_1_systems_status
    :field eps_1_boot_count: data.payload.eps_1_boot_count
    :field eps_2_mode: data.payload.eps_2_mode
    :field eps_2_consumption_current: data.payload.eps_2_consumption_current
    :field eps_2_solar_cells_current: data.payload.eps_2_solar_cells_current
    :field eps_2_cell_voltage_full: data.payload.eps_2_cell_voltage_full
    :field eps_2_battery_temperature: data.payload.eps_2_battery_temperature
    :field eps_2_temperature_sp_y_pos: data.payload.eps_2_temperature_sp_y_pos
    :field eps_2_temperature_sp_y_neg: data.payload.eps_2_temperature_sp_y_neg
    :field eps_2_temperature_sp_x_pos: data.payload.eps_2_temperature_sp_x_pos
    :field eps_2_temperature_sp_x_neg: data.payload.eps_2_temperature_sp_x_neg
    :field eps_2_systems_status: data.payload.eps_2_systems_status
    :field eps_2_boot_count: data.payload.eps_2_boot_count
    :field adcs_mt_mode: data.payload.adcs_mt_mode
    :field adcs_rm_mode: data.payload.adcs_rm_mode
    :field adcs_kf_mode: data.payload.adcs_kf_mode
    :field adcs_filter_reset_count: data.payload.adcs_filter_reset_count
    :field adcs_sensors_state: data.payload.adcs_sensors_state
    :field adcs_flywheel_state: data.payload.adcs_flywheel_state
    :field comm_type: data.payload.comm_type
    :field comm_vbus_voltage: data.payload.comm_vbus_voltage
    :field comm_boot_count: data.payload.comm_boot_count
    :field comm_rssi: data.payload.comm_rssi
    :field comm_rssi_minimal: data.payload.comm_rssi_minimal
    :field comm_received_valid_packets: data.payload.comm_received_valid_packets
    :field comm_received_invalid_packets: data.payload.comm_received_invalid_packets
    :field comm_sent_packets: data.payload.comm_sent_packets
    :field comm_status: data.payload.comm_status
    :field comm_mode: data.payload.comm_mode
    :field comm_amp_temperature: data.payload.comm_amp_temperature
    :field comm_reserved_1: data.payload.comm_reserved_1
    :field comm_reserved_2: data.payload.comm_reserved_2
    :field packet_id: data.data_header.packet_id
    :field packet_size: data.data_header.packet_size
    :field packet_info: data.data_header.packet_info
    :field cmd: data.data_payload.cmd
    :field gnss_timestamp: data.data_payload.payload.gnss_timestamp
    :field gnss_x: data.data_payload.payload.gnss_x
    :field gnss_y: data.data_payload.payload.gnss_y
    :field gnss_z: data.data_payload.payload.gnss_z
    :field gnss_v_x: data.data_payload.payload.gnss_v_x
    :field gnss_v_y: data.data_payload.payload.gnss_v_y
    :field gnss_v_z: data.data_payload.payload.gnss_v_z
    :field gnss_value_1: data.data_payload.payload.gnss_value_1
    :field gnss_value_2: data.data_payload.payload.gnss_value_2
    :field gnss_value_3: data.data_payload.payload.gnss_value_3
    :field gnss_value_4: data.data_payload.payload.gnss_value_4
    :field gnss_value_5: data.data_payload.payload.gnss_value_5
    :field gnss_value_6: data.data_payload.payload.gnss_value_6
    :field gnss_value_7: data.data_payload.payload.gnss_value_7
    :field gnss_value_8: data.data_payload.payload.gnss_value_8
    :field gnss_value_9: data.data_payload.payload.gnss_value_9
    :field gnss_value_10: data.data_payload.payload.gnss_value_10
    :field gnss_value_11: data.data_payload.payload.gnss_value_11
    :field gnss_value_12: data.data_payload.payload.gnss_value_12
    :field len: len
    :field type: type
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        _on = self.type
        if _on == 33930:
            self.data = Innosat16.Ax25Frame(self._io, self, self._root)
        else:
            self.data = Innosat16.DataFrame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Innosat16.Ax25Header(self._io, self, self._root)
            self.payload = Innosat16.GeoscanBeaconTlm(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Innosat16.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Innosat16.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Innosat16.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Innosat16.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()
            self.pid = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"BEACON") or (self.callsign == u"RS92S7")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class GnssTlm1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gnss_timestamp = self._io.read_u8le()
            self.gnss_x = self._io.read_f4le()
            self.gnss_y = self._io.read_f4le()
            self.gnss_z = self._io.read_f4le()
            self.gnss_v_x = self._io.read_f4le()
            self.gnss_v_y = self._io.read_f4le()
            self.gnss_v_z = self._io.read_f4le()
            self.gnss_value_1 = self._io.read_f4le()
            self.gnss_value_2 = self._io.read_f4le()
            self.gnss_value_3 = self._io.read_f4le()
            self.gnss_value_4 = self._io.read_f4le()
            self.gnss_value_5 = self._io.read_u1()
            self.gnss_value_6 = self._io.read_u1()
            self.gnss_value_7 = self._io.read_u1()
            self.gnss_value_8 = self._io.read_u1()
            self.gnss_value_9 = self._io.read_u2le()
            self.gnss_value_10 = self._io.read_f4le()
            self.gnss_value_11 = self._io.read_f4le()
            self.gnss_value_12 = self._io.read_f4le()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class DataHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_id = self._io.read_u2le()
            self.packet_size = self._io.read_u1()
            self.packet_info = self._io.read_u1()


    class DataTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cmd = self._io.read_u2le()
            _on = self.cmd
            if _on == 17085:
                self.payload = Innosat16.GnssTlm1(self._io, self, self._root)
            elif _on == 17341:
                self.payload = Innosat16.GnssTlm2(self._io, self, self._root)


    class GnssTlm2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gnss_timestamp = self._io.read_u4le()
            self.gnss_x = self._io.read_f4le()
            self.gnss_y = self._io.read_f4le()
            self.gnss_z = self._io.read_f4le()
            self.gnss_v_x = self._io.read_f4le()
            self.gnss_v_y = self._io.read_f4le()
            self.gnss_v_z = self._io.read_f4le()
            self.gnss_value_1 = self._io.read_f4le()
            self.gnss_value_2 = self._io.read_f4le()
            self.gnss_value_3 = self._io.read_f4le()
            self.gnss_value_4 = self._io.read_f4le()
            self.gnss_value_5 = self._io.read_f4le()
            self.gnss_value_6 = self._io.read_f4le()
            self.gnss_value_7 = self._io.read_f4le()
            self.gnss_value_8 = self._io.read_u2le()
            self.gnss_value_9 = self._io.read_u8le()


    class GeoscanBeaconTlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id = self._io.read_u1()
            self.eps_1_mode = self._io.read_u1()
            self.eps_1_consumption_current = self._io.read_u2le()
            self.eps_1_solar_cells_current = self._io.read_u2le()
            self.eps_1_cell_voltage_full = self._io.read_u2le()
            self.eps_1_battery_temperature = self._io.read_s1()
            self.eps_1_temperature_sp_y_pos = self._io.read_s1()
            self.eps_1_temperature_sp_y_neg = self._io.read_s1()
            self.eps_1_temperature_sp_x_pos = self._io.read_s1()
            self.eps_1_temperature_sp_x_neg = self._io.read_s1()
            self.eps_1_systems_status = self._io.read_u2le()
            self.eps_1_boot_count = self._io.read_u2le()
            self.eps_2_mode = self._io.read_u1()
            self.eps_2_consumption_current = self._io.read_u2le()
            self.eps_2_solar_cells_current = self._io.read_u2le()
            self.eps_2_cell_voltage_full = self._io.read_u2le()
            self.eps_2_battery_temperature = self._io.read_s1()
            self.eps_2_temperature_sp_y_pos = self._io.read_s1()
            self.eps_2_temperature_sp_y_neg = self._io.read_s1()
            self.eps_2_temperature_sp_x_pos = self._io.read_s1()
            self.eps_2_temperature_sp_x_neg = self._io.read_s1()
            self.eps_2_systems_status = self._io.read_u2le()
            self.eps_2_boot_count = self._io.read_u2le()
            self.adcs_mt_mode = self._io.read_u1()
            self.adcs_rm_mode = self._io.read_u1()
            self.adcs_kf_mode = self._io.read_u1()
            self.adcs_filter_reset_count = self._io.read_u1()
            self.adcs_sensors_state = self._io.read_u2le()
            self.adcs_flywheel_state = self._io.read_u1()
            self.comm_type = self._io.read_u1()
            self.comm_vbus_voltage = self._io.read_u2le()
            self.comm_boot_count = self._io.read_u2le()
            self.comm_rssi = self._io.read_s1()
            self.comm_rssi_minimal = self._io.read_s1()
            self.comm_received_valid_packets = self._io.read_u1()
            self.comm_received_invalid_packets = self._io.read_u1()
            self.comm_sent_packets = self._io.read_u1()
            self.comm_status = self._io.read_u1()
            self.comm_mode = self._io.read_u1()
            self.comm_amp_temperature = self._io.read_s1()
            self.comm_reserved_1 = self._io.read_u2le()
            self.comm_reserved_2 = self._io.read_u1()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Innosat16.Callsign(_io__raw_callsign_ror, self, self._root)


    class DataFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_header = Innosat16.DataHeader(self._io, self, self._root)
            self.data_payload = Innosat16.DataTlm(self._io, self, self._root)


    @property
    def len(self):
        if hasattr(self, '_m_len'):
            return self._m_len

        self._m_len = self._io.size()
        return getattr(self, '_m_len', None)

    @property
    def type(self):
        if hasattr(self, '_m_type'):
            return self._m_type

        _pos = self._io.pos()
        self._io.seek(0)
        self._m_type = self._io.read_u2be()
        self._io.seek(_pos)
        return getattr(self, '_m_type', None)





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Rhoksat(KaitaiStruct):
    """:field dest_callsign_raw_callsign: ax25_frame.ax25_header.dest_callsign_raw.dest_callsign_ror.dest_callsign
    :field dest_ssid_raw_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field dest_ssid_raw_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field dest_ssid_raw_value: ax25_frame.ax25_header.dest_ssid_raw.value
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.src_callsign_ror.src_callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field src_ssid_raw_value: ax25_frame.ax25_header.src_ssid_raw.value
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field beacon_id: ax25_frame.payload.beacon.beacon_id
    :field num_reboots: ax25_frame.payload.beacon.num_reboots
    :field bat_voltage: ax25_frame.payload.beacon.bat_voltage
    :field rate_x: ax25_frame.payload.beacon.imtq.rate_x
    :field rate_y: ax25_frame.payload.beacon.imtq.rate_y
    :field rate_z: ax25_frame.payload.beacon.imtq.rate_z
    :field total: ax25_frame.payload.beacon.imtq.total
    :field num_of_sweeps: ax25_frame.payload.beacon.num_of_sweeps
    :field num_of_logs: ax25_frame.payload.beacon.num_of_logs
    :field time: ax25_frame.payload.beacon.time
    :field sdcard_id: ax25_frame.payload.beacon.sdcard_id
    :field gyro_x_raw: ax25_frame.payload.beacon.gyro.gyro_x_raw
    :field gyro_y_raw: ax25_frame.payload.beacon.gyro.gyro_y_raw
    :field gyro_z_raw: ax25_frame.payload.beacon.gyro.gyro_z_raw
    :field acc_x_raw: ax25_frame.payload.beacon.accelerometer.acc_x_raw
    :field acc_y_raw: ax25_frame.payload.beacon.accelerometer.acc_y_raw
    :field acc_z_raw: ax25_frame.payload.beacon.accelerometer.acc_z_raw
    :field tl: ax25_frame.payload.beacon.sun_sensor.tl
    :field bl: ax25_frame.payload.beacon.sun_sensor.bl
    :field br: ax25_frame.payload.beacon.sun_sensor.br
    :field tr: ax25_frame.payload.beacon.sun_sensor.tr
    :field pd_z_minus: ax25_frame.payload.beacon.panels.pd_z_minus
    :field pd_x_minus: ax25_frame.payload.beacon.panels.pd_x_minus
    :field pd_x_plus: ax25_frame.payload.beacon.panels.pd_x_plus
    :field pd_y_plus: ax25_frame.payload.beacon.panels.pd_y_plus
    :field pd_y_minus: ax25_frame.payload.beacon.panels.pd_y_minus
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Rhoksat.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Rhoksat.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Rhoksat.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Rhoksat.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Rhoksat.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Rhoksat.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Rhoksat.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Rhoksat.IFrame(self._io, self, self._root)


    class DestCallsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Rhoksat.DestCallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Rhoksat.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Rhoksat.SrcCallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Rhoksat.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.beacon = Rhoksat.Beacon(self._io, self, self._root)


    class SrcCallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_src_callsign_ror = self._io.read_bytes(6)
            self._raw_src_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_src_callsign_ror, 8 - (1), 1)
            _io__raw_src_callsign_ror = KaitaiStream(BytesIO(self._raw_src_callsign_ror))
            self.src_callsign_ror = Rhoksat.SrcCallsign(_io__raw_src_callsign_ror, self, self._root)


    class Accelerometer(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.acc_x_raw = self._io.read_u2le()
            self.acc_y_raw = self._io.read_u2le()
            self.acc_z_raw = self._io.read_u2le()


    class Gyro(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gyro_x_raw = self._io.read_u2le()
            self.gyro_y_raw = self._io.read_u2le()
            self.gyro_z_raw = self._io.read_u2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.beacon = Rhoksat.Beacon(self._io, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class Beacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_id = self._io.read_u1()
            if not self.beacon_id == 253:
                raise kaitaistruct.ValidationNotEqualError(253, self.beacon_id, self._io, u"/types/beacon/seq/0")
            self.num_reboots = self._io.read_s4le()
            self.bat_voltage = self._io.read_s4le()
            self.imtq = Rhoksat.Imtq(self._io, self, self._root)
            self.num_of_sweeps = self._io.read_u4le()
            self.num_of_logs = self._io.read_u4le()
            self.time = self._io.read_u4le()
            self.sdcard_id = self._io.read_u1()
            if not  ((self.sdcard_id == 0) or (self.sdcard_id == 1)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.sdcard_id, self._io, u"/types/beacon/seq/7")
            self.gyro = Rhoksat.Gyro(self._io, self, self._root)
            self.accelerometer = Rhoksat.Accelerometer(self._io, self, self._root)
            self.sun_sensor = Rhoksat.SunSensor(self._io, self, self._root)
            self.panels = Rhoksat.Panels(self._io, self, self._root)


    class SunSensor(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.tl = self._io.read_f4le()
            self.bl = self._io.read_f4le()
            self.br = self._io.read_f4le()
            self.tr = self._io.read_f4le()


    class Panels(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pd_z_minus = self._io.read_u4le()
            self.pd_x_minus = self._io.read_u4le()
            self.pd_x_plus = self._io.read_u4le()
            self.pd_y_plus = self._io.read_u4le()
            self.pd_y_minus = self._io.read_u4le()


    class Imtq(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rate_x = self._io.read_u2le()
            self.rate_y = self._io.read_u2le()
            self.rate_z = self._io.read_u2le()
            self.total = self._io.read_u2le()


    class DestCallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_dest_callsign_ror = self._io.read_bytes(6)
            self._raw_dest_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_dest_callsign_ror, 8 - (1), 1)
            _io__raw_dest_callsign_ror = KaitaiStream(BytesIO(self._raw_dest_callsign_ror))
            self.dest_callsign_ror = Rhoksat.DestCallsign(_io__raw_dest_callsign_ror, self, self._root)


    class SrcCallsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.src_callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not self.src_callsign == u"WP2XJL":
                raise kaitaistruct.ValidationNotEqualError(u"WP2XJL", self.src_callsign, self._io, u"/types/src_callsign/seq/0")



